@@ -1,5 +1,5 @@
 package ExtUtils::HasCompiler;
-$ExtUtils::HasCompiler::VERSION = '0.008';
+$ExtUtils::HasCompiler::VERSION = '0.009';
 use strict;
 use warnings;
 
@@ -64,6 +64,7 @@ EXPORT XS_EXTERNAL(boot_%s) {
 END
 
 my $counter = 1;
+my %prelinking = map { $_ => 1 } qw/MSWin32 VMS aix/;
 
 sub can_compile_loadable_object {
 	my %args = @_;
@@ -81,26 +82,26 @@ sub can_compile_loadable_object {
 
 	my $abs_basename = catfile($tempdir, $basename);
 	my $object_file = $abs_basename . $config->get('_o');
-	my $loadable_object = $abs_basename . $config->get('dlext');
+	my $loadable_object = $abs_basename . '.' . $config->get('dlext');
 	my $incdir = catdir($config->get('archlibexp'), 'CORE');
 
 	my ($cc, $ccflags, $optimize, $cccdlflags, $ld, $lddlflags, $libperl, $perllibs) = map { $config->get($_) } qw/cc ccflags optimize cccdlflags ld lddlflags libperl perllibs/;
 
 	my @commands;
+	Mksymlists(NAME => $basename, FILE => $abs_basename) if $prelinking{$^O};
 	if ($^O eq 'MSWin32' && $cc =~ /^cl/) {
-		Mksymlists(NAME => $basename, FILE => $abs_basename);
 		push @commands, qq{$cc $ccflags $cccdlflags $optimize /I "$incdir" $source_name /Fo$object_file};
 		push @commands, qq{$ld $object_file $lddlflags $libperl $perllibs /out:$loadable_object /def:$abs_basename.def /pdb:$abs_basename.pdb};
 	}
 	elsif ($^O eq 'VMS') {
-		carp "VMS is currently unsupported";
-		return;
+		my $incdirs = $ccflags =~ s{ /inc[^=]+ (?:=)+ (?:\()? ( [^\/\)]* ) }{}xi ? "$1,$incdir" : $incdir;
+		push @commands, qq{$cc $ccflags $optimize /include=($incdirs) $cccdlflags $source_name /obj=$object_file};
+		push @commands, qq{$cc $optimize $lddlflags=$loadable_object $object_file,$abs_basename.opt/OPTIONS,${incdir}perlshr_attr.opt/OPTIONS' $perllibs};
 	}
 	else {
 		my @extra;
 		if ($^O eq 'MSWin32') {
 			push @extra, "$abs_basename.def";
-			Mksymlists(NAME => $basename, FILE => $abs_basename);
 			push @extra, '-l' . ($libperl =~ /lib([^.]+)\./)[0];
 		}
 		elsif ($^O eq 'cygwin') {
@@ -109,7 +110,6 @@ sub can_compile_loadable_object {
 		elsif ($^O eq 'aix') {
 			$lddlflags =~ s/\Q$(BASEEXT)\E/$abs_basename/;
 			$lddlflags =~ s/\Q$(PERL_INC)\E/$incdir/;
-			Mksymlists(NAME => $basename, FILE => $abs_basename);
 		}
 		push @commands, qq{$cc $ccflags $optimize "-I$incdir" $cccdlflags -c $source_name -o $object_file};
 		push @commands, qq{$cc $optimize $object_file -o $loadable_object $lddlflags @extra $perllibs};
@@ -128,12 +128,15 @@ sub can_compile_loadable_object {
 	if ($handle) {
 		my $symbol = DynaLoader::dl_find_symbol($handle, "boot_$basename");
 		my $compilet = DynaLoader::dl_install_xsub('__ANON__::__ANON__', $symbol, $source_name);
-		my $ret = eval { $compilet->(); $package->exported };
+		my $ret = eval { $compilet->(); $package->exported } or carp $@;
 		delete $ExtUtils::HasCompiler::{"$shortname\::"};
-		DynaLoader::dl_unload_file($handle);
+		eval { DynaLoader::dl_unload_file($handle) } or carp $@;
 		return $ret == 42;
 	}
-	return;
+	else {
+		carp "Couldn't load $loadable_object: " . DynaLoader::dl_error();
+		return;
+	}
 }
 
 sub ExtUtils::HasCompiler::Config::get {
@@ -157,7 +160,7 @@ ExtUtils::HasCompiler - Check for the presence of a compiler
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 DESCRIPTION
 
